using System;
using System.ComponentModel;
using System.ComponentModel.Design;
using System.Xml;
using System.Xml.XPath;
using System.IO;
using System.Reflection;
using System.Diagnostics;

namespace SAMS.ToolKit.Licensing
{
	public class CustomLicense : System.ComponentModel.License {
		private string					licenseKey = null;
		private CustomLicenseProvider	owner	= null;

		public CustomLicense( CustomLicenseProvider owner, string key ) {
			this.owner		= owner;
			this.licenseKey	= key;
		}

		public override string LicenseKey {
			get { return licenseKey; }
		}

		public override void Dispose( ) {
		}
	}

	public class CustomLicenseProvider : System.ComponentModel.LicenseProvider	{
		
		
		protected  bool IsKeyValid( string key, Type type ) {
			return (key == null ? false : true);
		}

		
		public override License GetLicense( LicenseContext context, 
			                                   Type type, 
			                                   object instance, 
											   bool allowExceptions ) {

			CustomLicense	license	= null;
			if( context == null ) return null;

			
			if( context.UsageMode == LicenseUsageMode.Runtime ) {
				//See if the key is stored
				string key = context.GetSavedLicenseKey( type, null );
				if( key != null && IsKeyValid( key, type ) )
					license = new CustomLicense( this, key );
			}

			if( license == null ) {
				//locate sams.license.xml file
				ITypeResolutionService resolver = (ITypeResolutionService)context.GetService( typeof( ITypeResolutionService ) );

				string location = resolver.GetPathOfAssembly( type.Assembly.GetName( ) );

				string licenseFile = Path.GetDirectoryName( location ) + "\\sams.license.xml";

				Debug.WriteLine( string.Format("License file location : {0}", licenseFile ) );

				
				try {
					XmlDocument	doc = new XmlDocument( );
					doc.Load( licenseFile );
					string xpathExpression = string.Format("descendant::control[@name='{0}']", type.FullName);
					XmlNode controlNode = doc.SelectSingleNode( xpathExpression );
					if( controlNode != null ) {
						//get the key value
						string key = controlNode.Attributes.GetNamedItem( "key" ).Value;
						if( key != null ) {
							license = new CustomLicense( this, key );
							//Save the license in the current context
							context.SetSavedLicenseKey( type, license.LicenseKey );
						}
					}
				} catch( Exception e ) {
					if( allowExceptions )
						throw new LicenseException( type, instance, "License not valid" );
					else
						return null;
				}
			}
			return license;
		}
	}
}
