using System;
using System.Drawing;
using System.Drawing.Design;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Windows.Forms.Design;


namespace SAMS.ToolKit.Design
{

	public class IconEditor : System.Drawing.Design.UITypeEditor 
	{
		
		public override object EditValue(ITypeDescriptorContext context, 
										 IServiceProvider provider, 
										 object value) {
			
			IconEditorDialog dlg = new IconEditorDialog( );
			IWindowsFormsEditorService winFormEditorService = (IWindowsFormsEditorService)provider.GetService( typeof( IWindowsFormsEditorService ) );
			if( DialogResult.OK == winFormEditorService.ShowDialog( dlg ) )
				value = dlg.SelectedIcon;
			
			return value;
		}

		public override UITypeEditorEditStyle GetEditStyle(ITypeDescriptorContext context) {
			return UITypeEditorEditStyle.Modal;
		}

		public override bool GetPaintValueSupported(ITypeDescriptorContext context) {
			return true;
		}

		public override void PaintValue(System.Drawing.Design.PaintValueEventArgs e) {
			if( !(e.Value is Icon) ) 
				return;
			Image img = ((Icon)e.Value).ToBitmap( );
			Rectangle rcBounds = e.Bounds;
			rcBounds.Inflate(-1,-1);
			Pen p = System.Drawing.SystemPens.WindowFrame;
			e.Graphics.DrawRectangle(p, rcBounds );
			if( img != null )
				e.Graphics.DrawImage( img, e.Bounds );
		}
		
				
		protected class IconEditorDialog : System.Windows.Forms.Form 
		{
			private System.Windows.Forms.PictureBox iconPreviewPictureBox;
			private System.Windows.Forms.Button btnOpen;
			private System.Windows.Forms.Button btnOK;
			private System.Windows.Forms.Button btnCancel;
			private System.Windows.Forms.PictureBox samsImagePictureBox;
			private System.Windows.Forms.OpenFileDialog openFileDialog;
		
			private System.ComponentModel.Container components = null;

			private System.Drawing.Icon	selectedIcon;

			public Icon SelectedIcon 
			{
				get { return selectedIcon; }
				set 
				{
					selectedIcon = value;
					iconPreviewPictureBox.Image = ( selectedIcon != null ? selectedIcon.ToBitmap( ) : null );
				}
			}


			public IconEditorDialog() 
			{
				InitializeComponent();
			}

			/// <summary>
			/// Clean up any resources being used.
			/// </summary>
			protected override void Dispose( bool disposing ) 
			{
				if( disposing ) 
				{
					if (components != null) 
					{
						components.Dispose();
					}
				}
				base.Dispose( disposing );
			}

		#region Windows Form Designer generated code
			/// <summary>
			/// Required method for Designer support - do not modify
			/// the contents of this method with the code editor.
			/// </summary>
			private void InitializeComponent() 
			{
				//TODO: FIX RESOURCE REFERENCE
				//System.Resources.ResourceManager resources = new System.Resources.ResourceManager(typeof(IconEditorDialog));
				this.openFileDialog = new System.Windows.Forms.OpenFileDialog();
				this.btnCancel = new System.Windows.Forms.Button();
				this.iconPreviewPictureBox = new System.Windows.Forms.PictureBox();
				this.btnOpen = new System.Windows.Forms.Button();
				this.samsImagePictureBox = new System.Windows.Forms.PictureBox();
				this.btnOK = new System.Windows.Forms.Button();
				this.SuspendLayout();
				// 
				// openFileDialog
				// 
				this.openFileDialog.Filter = "Icon Files | *.ico";
				// 
				// btnCancel
				// 
				this.btnCancel.Location = new System.Drawing.Point(120, 88);
				this.btnCancel.Name = "btnCancel";
				this.btnCancel.Size = new System.Drawing.Size(88, 24);
				this.btnCancel.TabIndex = 2;
				this.btnCancel.Text = "Cancel";
				this.btnCancel.Click += new System.EventHandler(this.btnCancel_Click);
				// 
				// iconPreviewPictureBox
				// 
				this.iconPreviewPictureBox.BorderStyle = System.Windows.Forms.BorderStyle.Fixed3D;
				this.iconPreviewPictureBox.Location = new System.Drawing.Point(8, 8);
				this.iconPreviewPictureBox.Name = "iconPreviewPictureBox";
				this.iconPreviewPictureBox.Size = new System.Drawing.Size(100, 68);
				this.iconPreviewPictureBox.SizeMode = System.Windows.Forms.PictureBoxSizeMode.CenterImage;
				this.iconPreviewPictureBox.TabIndex = 1;
				this.iconPreviewPictureBox.TabStop = false;
				// 
				// btnOpen
				// 
				this.btnOpen.Location = new System.Drawing.Point(120, 8);
				this.btnOpen.Name = "btnOpen";
				this.btnOpen.Size = new System.Drawing.Size(88, 24);
				this.btnOpen.TabIndex = 2;
				this.btnOpen.Text = "Open...";
				this.btnOpen.Click += new System.EventHandler(this.btnOpen_Click);
				// 
				// samsImagePictureBox
				//TODO: FIX 
				//this.samsImagePictureBox.Image = ((System.Drawing.Bitmap)(resources.GetObject("samsImagePictureBox.Image")));
				this.samsImagePictureBox.Location = new System.Drawing.Point(8, 88);
				this.samsImagePictureBox.Name = "samsImagePictureBox";
				this.samsImagePictureBox.Size = new System.Drawing.Size(96, 32);
				this.samsImagePictureBox.SizeMode = System.Windows.Forms.PictureBoxSizeMode.CenterImage;
				this.samsImagePictureBox.TabIndex = 3;
				this.samsImagePictureBox.TabStop = false;
				// 
				// btnOK
				// 
				this.btnOK.Location = new System.Drawing.Point(120, 48);
				this.btnOK.Name = "btnOK";
				this.btnOK.Size = new System.Drawing.Size(88, 24);
				this.btnOK.TabIndex = 2;
				this.btnOK.Text = "OK";
				this.btnOK.Click += new System.EventHandler(this.btnOK_Click);
				// 
				// IconEditorDialog
				// 
				this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
				this.ClientSize = new System.Drawing.Size(228, 129);
				this.Controls.AddRange(new System.Windows.Forms.Control[] {
																			  this.samsImagePictureBox,
																			  this.btnCancel,
																			  this.btnOK,
																			  this.btnOpen,
																			  this.iconPreviewPictureBox});
				this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.FixedDialog;
				this.Name = "IconEditorDialog";
				this.Text = "IconButton UITypeEditor";
				this.ResumeLayout(false);

			}
		#endregion

			private void btnOpen_Click(object sender, System.EventArgs e) 
			{
				if( DialogResult.OK == openFileDialog.ShowDialog( ) ) 
				{
					try 
					{
						selectedIcon = new Icon( openFileDialog.FileName );
					}
					catch( Exception exception ) 
					{
						MessageBox.Show( this, exception.Message );
					} 
					finally 
					{
						iconPreviewPictureBox.Image = ( selectedIcon != null ? selectedIcon.ToBitmap( ) : null );
					}
				}
			}

			private void btnOK_Click(object sender, System.EventArgs e) 
			{
				DialogResult = DialogResult.OK;
				Close( );
			}

			private void btnCancel_Click(object sender, System.EventArgs e) 
			{
				DialogResult = DialogResult.Cancel;
				Close( );
			}
		}
	}
}
