Imports System.Windows.Forms
Imports System.Drawing

Namespace SAMS.ToolKit.VB.Controls

    <System.ComponentModel.Description("SAMS IconButton Control (VB)")> _
    Public Class IconButton
        Inherits System.Windows.Forms.Control

#Region "Const Member Fields"
        Protected Const EDGE_PADDING As Integer = 4
#End Region

#Region "Implementation Member Fields"
        Protected buttonState As System.Windows.Forms.buttonState
        Protected buttonIcon As System.Drawing.Icon
        Protected mousePressed As Boolean
        Protected iconDrawWidth As Integer
#End Region

#Region "Properties"

        <System.ComponentModel.Description("The Icon to be displayed in the button"), _
        System.ComponentModel.Category("Appearance")> _
        Public Property Icon() As Icon
            Get
                Icon = buttonIcon
            End Get
            Set(ByVal value As Icon)
                buttonIcon = value
                Invalidate()
                Update()
            End Set
        End Property
#End Region


#Region "Construction"

        Public Sub IconButton()
            buttonState = buttonState.Normal
            buttonIcon = Nothing
            iconDrawWidth = 0
        End Sub

#End Region

#Region "Control Overrides"

        Protected Overrides Sub OnMouseDown(ByVal e As System.Windows.Forms.MouseEventArgs)
            If (e.Button = MouseButtons.Left) Then
                Focus()
                Capture = True
                buttonState = buttonState.Pushed
                mousePressed = True
                Invalidate()
                Update()
            Else
                MyBase.OnMouseDown(e)
            End If
        End Sub


        Protected Overrides Sub OnMouseUp(ByVal e As System.Windows.Forms.MouseEventArgs)
            If (mousePressed And (e.Button = MouseButtons.Left)) Then
                Capture = False
                mousePressed = False
                buttonState = buttonState.Normal
                Invalidate()
                Update()
            Else
                MyBase.OnMouseUp(e)
            End If
        End Sub

        Protected Overrides Sub OnPaint(ByVal e As System.Windows.Forms.PaintEventArgs)
            Draw(e.Graphics)
            MyBase.OnPaint(e)
        End Sub

        Protected Overrides Sub OnSizeChanged(ByVal e As System.EventArgs)
            Invalidate()
            MyBase.OnSizeChanged(e)
        End Sub

        Protected Overrides Sub OnTextChanged(ByVal e As System.EventArgs)
            Invalidate()
        End Sub

        Protected Overrides Sub OnLostFocus(ByVal e As System.EventArgs)
            Invalidate()
        End Sub

        Protected Overrides Sub OnGotFocus(ByVal e As System.EventArgs)
            Invalidate()
        End Sub

#End Region

#Region "Implementation"
        Protected Overridable Sub Draw(ByRef g As System.Drawing.Graphics)
            DrawButton(g)
            If (Not buttonIcon Is Nothing) Then
                DrawIcon(g)
            End If
            DrawText(g)

            If (MyBase.Focused) Then
                DrawFocusClues(g)
            End If
        End Sub

        Protected Overridable Sub DrawButton(ByRef g As System.Drawing.Graphics)
            Dim rcButton As New Rectangle(0, 0, Width, Height)
            If (Focused) Then
                rcButton.Inflate(-1, -1)
            End If
            ControlPaint.DrawButton(g, rcButton, buttonState)
        End Sub

        Protected Overridable Sub DrawIcon(ByRef g As System.Drawing.Graphics)
            System.Diagnostics.Debug.Assert(Not buttonIcon Is Nothing, "IconButton is null")

            Dim top, height, width As Integer

            top = (Me.Height / 2) - (buttonIcon.Height / 2)
            height = buttonIcon.Height
            width = buttonIcon.Width

            If ((top + height) >= Me.Height) Then
                'Scale the image to fit in (w,h) of button
                top = EDGE_PADDING
                Dim drawHeight As Integer = Me.Height - (2 * EDGE_PADDING)
                Dim scale As Single = drawHeight / height
                width = width * scale
                height = drawHeight
            End If

            Dim iconRect As New Rectangle(EDGE_PADDING, top, width, height)
            If (buttonState.Pushed = buttonState) Then
                iconRect.Offset(1F, 1F)
            End If

            g.DrawIcon(buttonIcon, iconRect)
            iconDrawWidth = iconRect.Width
        End Sub

        Protected Overridable Sub DrawText(ByRef g As System.Drawing.Graphics)
            Dim top, left, width, height As Integer

            top = EDGE_PADDING
            left = IIf(buttonIcon Is Nothing, EDGE_PADDING, iconDrawWidth + EDGE_PADDING)
            width = Me.Width - left
            height = Me.Height - (2 * EDGE_PADDING)

            Dim layoutRect As New RectangleF(left, top, width, height)
            If (buttonState.Pushed = buttonState) Then
                layoutRect.Offset(1F, 1F)
            End If

            Dim fmt As New StringFormat()
            fmt.Alignment = StringAlignment.Center
            fmt.LineAlignment = StringAlignment.Center

            Dim textBrush As New SolidBrush(Me.ForeColor)

            g.DrawString(Text, Font, textBrush, layoutRect, fmt)

            textBrush.Dispose()

        End Sub

        Protected Overridable Sub DrawFocusClues(ByRef g As System.Drawing.Graphics)
            Dim p As New System.Drawing.Pen(System.Drawing.Color.Black, 1F)
            Dim frameRect = New Rectangle(0, 0, Me.Width, Me.Height)
            g.DrawRectangle(p, frameRect)

            p.DashStyle = Drawing.Drawing2D.DashStyle.Dot
            frameRect = New Rectangle(2, 2, Me.Width - 6, Me.Height - 6)
            If (buttonState.Pushed = buttonState) Then
                frameRect.offset(1, 1)
            End If
            g.DrawRectangle(p, frameRect)

            p.Dispose()
        End Sub
#End Region

    End Class

End Namespace