using System;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;

namespace FontDemo
{
	
	public class FontProperties {
		private Font		theFont;
		private Color		fontColor;
		private Brush		theFontBrush;
		private Image		theImage;

		private Color		backgroundColor;
		private Image		backgroundImage;
		private Brush		backgroundBrush;

		private Color		shadowColor;
		private Brush		theShadowBrush;
		private int			shadowOffset;
		private int			shadowOpacity;
		private float		rotation;

		private string		text;

		public event EventHandler PropertyChanged;

		
		public Font Font {
			get { return theFont; }
			set { 
				theFont = value;
				OnPropertyChanged( new EventArgs( ) );
			}
		}

		public Color FontColor {
			get { return fontColor; }
			set {
				fontColor = value;
				theFontBrush = new SolidBrush( fontColor );
				OnPropertyChanged( new EventArgs( ) );
			}
		}

		public Color ShadowColor {
			get { return shadowColor; }
			set {
				shadowColor = value;
				theShadowBrush = new SolidBrush( Color.FromArgb( shadowOpacity, shadowColor ) );
				OnPropertyChanged( new EventArgs( ) );
			}
		}

		public int ShadowOpacity {
			get { return shadowOpacity; }
			set {
				shadowOpacity = value;
				theShadowBrush = new SolidBrush( Color.FromArgb( shadowOpacity, shadowColor ) );
				OnPropertyChanged( new EventArgs( ) );
			}
		}
		
		public int ShadowOffset {
			get { return shadowOffset; }
			set { 
				shadowOffset = value;
				OnPropertyChanged( new EventArgs( ) );
			}
		}
		public Image Image {
			get { return theImage; }
			set {
				theImage = value;
				theFontBrush = new TextureBrush( theImage );
				OnPropertyChanged( new EventArgs( ) );
			}
		}

		public Color BackColor {
			get { return backgroundColor; }
			set {
				backgroundColor = value;
				backgroundBrush = new SolidBrush( backgroundColor );
				OnPropertyChanged( new EventArgs( ) );
			}
		}
		
		public Image BackgroundImage {
			get { return backgroundImage; }
			set {
				backgroundImage = value; 
				if( backgroundImage != null )
					backgroundBrush = new TextureBrush( backgroundImage );
				else
					backgroundBrush = new SolidBrush( backgroundColor );	
				OnPropertyChanged( new EventArgs( ) );
			}
		}

		public string Text {
			get { return text; }
			set {
				text = value;
				OnPropertyChanged( new EventArgs( ) );
			}
		}
		
		public float Rotation {
			get { return rotation; }
			set {
				rotation = value;
				OnPropertyChanged( new EventArgs( ) );
			}
		}
		

			//Properties to be used by Font Demo
			//The BrowsableAttribute is used by the Property
			//Grid to determine if the property should be
			//accessable or not.
		
			[System.ComponentModel.Browsable( false )]
			public Brush FontBrush {
			get { return theFontBrush; }
			set { theFontBrush = value; }
		}

		[System.ComponentModel.Browsable( false )]
		public Brush ShadowBrush {
			get { return theShadowBrush; }
			set { theShadowBrush = value; }
		}

		[System.ComponentModel.Browsable( false )]
		public Brush BackgroundBrush {
			get { return backgroundBrush; }
			set { backgroundBrush = value; }
		}
		
		protected void OnPropertyChanged( EventArgs e ) {
			if( PropertyChanged != null )
				PropertyChanged( this, e );
		}
	}

			



	public class Form1 : System.Windows.Forms.Form
	{
		private FontProperties						fontProperties;
		private System.Windows.Forms.PropertyGrid	propertyGrid1;
		private System.Windows.Forms.Splitter		splitter1;
		/// <summary>
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;


		public Form1()
		{
			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();

			this.SetStyle( ControlStyles.ResizeRedraw, true );

			fontProperties = new FontProperties( );
			
			//set default values
			fontProperties.Font = (Font)this.Font.Clone( );
			fontProperties.FontColor = Color.Black;
			fontProperties.ShadowColor = Color.LightGray;
			fontProperties.ShadowOpacity = 100; //range 0 to 255
			fontProperties.BackgroundBrush = new SolidBrush( Color.White );
			fontProperties.Text = "Hello Fonts!";
			fontProperties.PropertyChanged += new EventHandler( this.OnFPPropertyChangedEventHandler );
			propertyGrid1.SelectedObject = fontProperties;
			propertyGrid1.Refresh( );

			
			this.Invalidate( );

		}

		

		protected void OnFPPropertyChangedEventHandler( object sender, EventArgs e ) {
			this.Invalidate( );
		}

		protected override void OnPaint( PaintEventArgs e ) {
			Rectangle drawZone = new Rectangle( DisplayRectangle.Left + propertyGrid1.Width,
												DisplayRectangle.Top,
												DisplayRectangle.Width - propertyGrid1.Width,
												DisplayRectangle.Height );

			//Fill the background image
			e.Graphics.FillRectangle( fontProperties.BackgroundBrush, drawZone );
			

			
			SizeF stringSize = e.Graphics.MeasureString( fontProperties.Text, fontProperties.Font );

			
			System.Drawing.Drawing2D.GraphicsState gs = e.Graphics.Save( );
			
			//Center the Text
			Point ptCenter   = new Point(drawZone.X + (drawZone.Width / 2), drawZone.Height / 2 );

			//Set the Transform to the center of the drawZone
			e.Graphics.TranslateTransform(ptCenter.X, ptCenter.Y );

			e.Graphics.RotateTransform( fontProperties.Rotation );
			
			//The Cordniate system now stems from the center of the DrawZone

			Point ptText = new Point( );
			ptText.X = -(int)(stringSize.Width / 2);
			ptText.Y = -(int)(stringSize.Height / 2);


			//Draw the Shadowed Text
			int offset = fontProperties.ShadowOffset;
			e.Graphics.DrawString( fontProperties.Text, 
								   fontProperties.Font, 
								   fontProperties.ShadowBrush, new Point(ptText.X + offset,ptText.Y + offset) );
			
			e.Graphics.DrawString( fontProperties.Text, 
								   fontProperties.Font, 
								   fontProperties.FontBrush, ptText);

			
			e.Graphics.Restore( gs );

		}
		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if (components != null) 
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.splitter1 = new System.Windows.Forms.Splitter();
			this.propertyGrid1 = new System.Windows.Forms.PropertyGrid();
			this.SuspendLayout();
			// 
			// splitter1
			// 
			this.splitter1.Location = new System.Drawing.Point(160, 0);
			this.splitter1.Name = "splitter1";
			this.splitter1.Size = new System.Drawing.Size(3, 341);
			this.splitter1.TabIndex = 1;
			this.splitter1.TabStop = false;
			this.splitter1.SplitterMoved += new System.Windows.Forms.SplitterEventHandler(this.splitter1_SplitterMoved);
			// 
			// propertyGrid1
			// 
			this.propertyGrid1.CommandsBackColor = System.Drawing.SystemColors.Window;
			this.propertyGrid1.CommandsVisibleIfAvailable = true;
			this.propertyGrid1.Dock = System.Windows.Forms.DockStyle.Left;
			this.propertyGrid1.LargeButtons = false;
			this.propertyGrid1.LineColor = System.Drawing.SystemColors.ScrollBar;
			this.propertyGrid1.Name = "propertyGrid1";
			this.propertyGrid1.Size = new System.Drawing.Size(160, 341);
			this.propertyGrid1.TabIndex = 0;
			this.propertyGrid1.Text = "propertyGrid1";
			this.propertyGrid1.ViewBackColor = System.Drawing.SystemColors.Window;
			this.propertyGrid1.ViewForeColor = System.Drawing.SystemColors.WindowText;
			// 
			// Form1
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.BackColor = System.Drawing.SystemColors.Window;
			this.ClientSize = new System.Drawing.Size(528, 341);
			this.Controls.AddRange(new System.Windows.Forms.Control[] {
																		  this.splitter1,
																		  this.propertyGrid1});
			this.Name = "Form1";
			this.Text = "Font Demo";
			this.ResumeLayout(false);

		}
		#endregion

		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main() 
		{
			Application.Run(new Form1());
		}

		private void splitter1_SplitterMoved(object sender, System.Windows.Forms.SplitterEventArgs e) {
			this.Invalidate( );
		}
	}
}
