using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;

namespace Pen_Demo
{

	//Used by the property grid
	public class PenProperties {

		//Implementation members
		private System.Drawing.Pen					thePen;
		
		//Events
		public event EventHandler PropertyChanged;

		//Properties
		public System.Drawing.Color Color {
			get { return thePen.Color; }
			set {
				thePen.Color = value;
				OnPropertyChanged( new EventArgs( ) );
			}
		}

		public System.Drawing.Drawing2D.DashCap DashCap {
			get { return thePen.DashCap; }
			set { 
				thePen.DashCap = value;
				OnPropertyChanged( new EventArgs( ) );
			}
		}

		public System.Drawing.Drawing2D.DashStyle DashStyle {
			get { return thePen.DashStyle; }
			set {
				thePen.DashStyle = value;
				OnPropertyChanged( new EventArgs( ) );
			}
		}

		public float Width {
			get { return thePen.Width; }
			set {
				thePen.Width = value;
				OnPropertyChanged( new EventArgs( ) );
			}
		}

		public float DashOffset {
			get { return thePen.DashOffset; }
			set {
				thePen.DashOffset = value;
				OnPropertyChanged( new EventArgs( ) );
			}
		}
				
		public System.Drawing.Drawing2D.LineCap StartCap {
			get { return thePen.StartCap; }
			set {
				thePen.StartCap = value;
				OnPropertyChanged( new EventArgs( ) );
			}
		}

		public System.Drawing.Drawing2D.LineCap EndCap {
			get { return thePen.EndCap; }
			set {
				thePen.EndCap = value;
				OnPropertyChanged( new EventArgs( ) );
			}
		}

		public PenProperties( Pen p ) {
			thePen = p;
		}

		protected void OnPropertyChanged( EventArgs e ) {
			if( PropertyChanged != null )
				PropertyChanged( this, e );
		}
	}

	/// <summary>
	/// Summary description for Form1.
	/// </summary>
	public class Form1 : System.Windows.Forms.Form
	{
		/// <summary>
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container		components = null;
		private System.Windows.Forms.PropertyGrid	propertyGrid1;
		private System.Drawing.Pen					thePen = null;
		private PenProperties					    penProps = null;

		public Form1()
		{
			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();

			thePen = new Pen( System.Drawing.Color.Black );
			penProps = new PenProperties( thePen );
			penProps.PropertyChanged += new EventHandler( this.OnPenPropertyChangedEventHandler );
			propertyGrid1.SelectedObject = penProps;
			propertyGrid1.Refresh( );
		
						
			
		}

		protected  override void OnSizeChanged( EventArgs e ) {
			base.OnSizeChanged( e );
			this.Invalidate( );
		}


		protected override void OnPaint( PaintEventArgs e ) {
			base.OnPaint( e );
			Rectangle drawZone = new Rectangle( this.ClientRectangle.Left, 
												this.ClientRectangle.Top, 
												this.ClientRectangle.Width, 
												this.ClientRectangle.Height );

			//Offset for Property Grid
			drawZone.X += this.propertyGrid1.Width;
			
			
			drawZone.Inflate(-10,-10);

			//Draw horizontal line
			e.Graphics.DrawLine( thePen, drawZone.Left, drawZone.Top, drawZone.Width, drawZone.Top );
			
			//Draw Vertical line
			e.Graphics.DrawLine( thePen, drawZone.Left + (thePen.Width/2), drawZone.Top + thePen.Width + 5, drawZone.Left + (thePen.Width/2), drawZone.Bottom  );
			
			//Draw Diag Line
			e.Graphics.DrawLine( thePen, drawZone.Left + (2*thePen.Width) , drawZone.Top + thePen.Width + 5, drawZone.Width , drawZone.Bottom  );
		}

		protected void OnPenPropertyChangedEventHandler( object sender, EventArgs e ) {
			this.Invalidate( );
		}



		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if (components != null) 
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.propertyGrid1 = new System.Windows.Forms.PropertyGrid();
			this.SuspendLayout();
			// 
			// propertyGrid1
			// 
			this.propertyGrid1.CommandsBackColor = System.Drawing.SystemColors.Window;
			this.propertyGrid1.CommandsVisibleIfAvailable = true;
			this.propertyGrid1.Dock = System.Windows.Forms.DockStyle.Left;
			this.propertyGrid1.LargeButtons = false;
			this.propertyGrid1.LineColor = System.Drawing.SystemColors.ScrollBar;
			this.propertyGrid1.Name = "propertyGrid1";
			this.propertyGrid1.Size = new System.Drawing.Size(160, 357);
			this.propertyGrid1.TabIndex = 0;
			this.propertyGrid1.Text = "propertyGrid1";
			this.propertyGrid1.ViewBackColor = System.Drawing.SystemColors.Window;
			this.propertyGrid1.ViewForeColor = System.Drawing.SystemColors.WindowText;
			// 
			// Form1
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.BackColor = System.Drawing.SystemColors.Window;
			this.ClientSize = new System.Drawing.Size(520, 357);
			this.Controls.AddRange(new System.Windows.Forms.Control[] {
																		  this.propertyGrid1});
			this.Name = "Form1";
			this.Text = "Pen Demo";
			this.ResumeLayout(false);

		}
		#endregion

		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main() 
		{
			Application.Run(new Form1());
		}
	}
}
