////////////////////////////////////////////////////////////////////////
///File		:IconButton.cs
///Author	:Richard L. Weeks
///
/// Copyright (c) 2001 by Richard L. Weeks
/// This file is provided for instructional purposes only.  
/// No warranties.
////////////////////////////////////////////////////////////////////////

using System;
using System.ComponentModel;
using System.ComponentModel.Design;
using System.Collections;
using System.Drawing;


namespace SAMS.ToolKit.Design
{
	/// <summary>
	/// Simple Designer for IconButton
	/// </summary>
	public class IconButtonDesigner : System.Windows.Forms.Design.ControlDesigner {
	
		private enum VERBS {
			Red,
			Green,
			Blue
		}

		private DesignerVerb[]			designerVerbs;

		public IconButtonDesigner()	{
			designerVerbs = new DesignerVerb[3];
			DesignerVerb[] verbs = new DesignerVerb[3];
			designerVerbs[(int)VERBS.Red] = new DesignerVerb( "Red", new EventHandler( this.OnRedVerb ) );
			designerVerbs[(int)VERBS.Green] = new DesignerVerb( "Green",  new EventHandler( this.OnGreenVerb ) );
			designerVerbs[(int)VERBS.Blue] = new DesignerVerb( "Blue",  new EventHandler( this.OnBlueVerb ) );
		}

		public override DesignerVerbCollection Verbs {
			get {
				return new DesignerVerbCollection( designerVerbs );
			}
		}


		//Overrides

		/// <summary>
		/// Remove some basic properties that are not supported by the IconButton
		/// </summary>
		/// <param name="Properties"></param>
		protected override void PostFilterProperties( IDictionary Properties ) {
			Properties.Remove( "BackgroundImage" );
			Properties.Remove( "BackColor" );
		}


		//Verb Handlers
		protected void OnRedVerb( object sender, EventArgs e ) {
			this.Control.ForeColor = System.Drawing.Color.Red;
			UpdateCheckMarks( VERBS.Red );
		}
		protected void OnGreenVerb( object sender, EventArgs e ) {
			this.Control.ForeColor =  System.Drawing.Color.Green;
			UpdateCheckMarks( VERBS.Green );
		}
		protected void OnBlueVerb( object sender, EventArgs e ) {
			this.Control.ForeColor =  System.Drawing.Color.Blue;
			UpdateCheckMarks( VERBS.Blue );
		}

		
		private void UpdateCheckMarks( VERBS ActiveVerb ) {
			foreach( DesignerVerb dv in designerVerbs )
				dv.Checked = false;
			designerVerbs[ (int)ActiveVerb ].Checked = true;
		}
	}
}
