using System;
using System.Drawing;
using System.Windows.Forms;


namespace SAMS.ToolKit.Controls
{
	public enum ScrollButtonOrientation {
		Up,
		Down
	}

	public class ScrollButton {

		private static int	SIZE			= 16;
		private static int  PADDING			= 5;
		private	Rectangle					btnRect;
		private ScrollButtonOrientation		orientation;

		public Point Location {
			get { return new Point( btnRect.X, btnRect.Y ); }
			set {
				btnRect = new Rectangle( ((Point)value).X, ((Point)value).Y, SIZE, SIZE );
			}
		}

		public ScrollButton( ScrollButtonOrientation sbo ) {
			orientation = sbo;
			
		}

		public bool HitTest( Point pt ) {
			return btnRect.Contains( pt );
		}

		public void Draw( Graphics g, ButtonState state ) {
			DrawButton( g, state );
			DrawArrow( g, state );
		}

		private void DrawButton( Graphics g, ButtonState state ) {
			if( btnRect.IsEmpty ) return;
			ControlPaint.DrawButton( g, btnRect, state );
		}

		private void DrawArrow( Graphics g, ButtonState state ) {
			if( orientation == ScrollButtonOrientation.Up )
				DrawUpArrow( g, state );
			else
				DrawDownArrow( g, state );
		}

		
		private void DrawUpArrow( Graphics g, ButtonState state ) {
			Point[] pts = new Point[3];
			pts[0].X = btnRect.Left + (btnRect.Width / 2) + (state == ButtonState.Pushed ? 1 : 0);
			pts[0].Y = btnRect.Top + PADDING + (state == ButtonState.Pushed ? 1 : 0);

			pts[1].X = btnRect.Left + 2 + (state == ButtonState.Pushed ? 1 : 0);
			pts[1].Y = btnRect.Bottom - PADDING + (state == ButtonState.Pushed ? 1 : 0);

			pts[2].X = btnRect.Right - 2 + (state == ButtonState.Pushed ? 1 : 0);
			pts[2].Y = btnRect.Bottom - PADDING + (state == ButtonState.Pushed ? 1 : 0);
			RenderArrow( g, pts );
		}

		private void DrawDownArrow( Graphics g, ButtonState state ) {
			Point[] pts = new Point[3];
			pts[0].X = btnRect.Left + (btnRect.Width / 2) + (state == ButtonState.Pushed ? 1 : 0);
			pts[0].Y = btnRect.Bottom - PADDING + (state == ButtonState.Pushed ? 1 : 0);

			pts[1].X = btnRect.Left + 2 + (state == ButtonState.Pushed ? 1 : 0);
			pts[1].Y = btnRect.Top + PADDING + (state == ButtonState.Pushed ? 1 : 0);

			pts[2].X = btnRect.Right - 2 + (state == ButtonState.Pushed ? 1 : 0);
			pts[2].Y = btnRect.Top + PADDING + (state == ButtonState.Pushed ? 1 : 0);

			RenderArrow( g, pts );
		}

		private void RenderArrow( Graphics g, Point[] pts ) {
			System.Drawing.Drawing2D.GraphicsPath path = new System.Drawing.Drawing2D.GraphicsPath( );
			path.AddLines( pts );
			path.CloseFigure( );
			Brush blackBrush = new SolidBrush(	System.Drawing.Color.Black );
			g.FillPath( blackBrush, path );
			blackBrush.Dispose( );
		}
	}
}
